#ifndef CONTROLLER_H
#define CONTROLLER_H

#include <QObject>
#include <QProcess>
#include <QSqlRecord>

class QWidget;

#include "qmlapplicationviewer.h"

class Controller : public QObject {
    Q_OBJECT

public:
    explicit Controller(QObject *parent = 0);

    void setView(QmlApplicationViewer* view);

public slots:

    QString getLanguage() const;
    void setOrientation(const QString &orient);
    QStringList getInstalledMediaPlayers() const;
    bool xTubeInstalled();
    void setMediaPlayer(const QString &player);
    void getMediaPlayerFromDB();
    QStringList getProxyFromDB() const;
    void toggleState();
    void minimize();
    void playVideo(const QString &url);
    void deleteVideo(const QString &path);
    void convertToAudio(const QString &input);
    QString archiveFileExists(const QString &filePath, const QString &archivePath) const;
    bool pathExists(const QString &path);
    void copyToClipboard(const QString &url);

private slots:
    void conversionFinished(int exitCode, QProcess::ExitStatus exitStatus);

private:
    QmlApplicationViewer *m_view;
    QString mediaPlayer;
    QProcess *converter;
    QString inputFile;
    QString outputFile;

signals:
    void alert(const QString &message);
    void conversionStarted();
    void conversionCompleted();
    void conversionFailed();
    void playbackStarted();
};

#endif // CONTROLLER_H
